import { For, createEffect, createSignal } from "solid-js";
import { DownloadIcon } from "../../icons/DownloadIcon";
import { Person } from "../../types/Person";
import { Register } from "../../types/Register";
import { courseMap } from "../../utils/allCourses";
import { certGenerator } from "../../certGenerator";

export function Registers(props: {person: Person | null}) {
    const [registers, setRegisters] = createSignal<Array<Register>>([]);

    createEffect(async() => {
        if (props.person === null) {
            setRegisters([]);
            return;
        }

        const person = props.person;

        const res = await fetch(`${import.meta.env.VITE_BACKEND_URL}/api/register/${person.person_dni}`);
        if (res.ok) {
            const data = await res.json();
            setRegisters(data);
        }
    });

    return (
        <div class="m-4 p-4 rounded-md">
            <h3 class="text-xl font-medium py-2">
                {props.person?.person_names}&nbsp;
                {props.person?.person_paternal_surname}&nbsp;
                {props.person?.person_maternal_surname}
            </h3>
            <div class="flex flex-wrap justify-start gap-2">
                <For each={registers()}>
                    {(register) => <RegisterEl register={register} person={props.person!} />}
                </For>
            </div>
        </div>
    );
}

function RegisterEl(props: {register: Register, person: Person}) {

    const dateComponents = () => {
        const [, year, month, day] = /(\d{4})-(\d{2})-(\d{2})/.exec(props.register.register_display_date) ?? [];
        return {year, month, day};
    };

    const displayDate = () => {
        const {year, month, day} = dateComponents();
        return `${day}/${month}/${year}`;
    };

    const courseName = () => {
        const course = courseMap()[props.register.register_course_id];
        return course?.course_name ?? "Curso no encontrado";
    };

    const genCert = () => {
        const courseN = courseName();
        const person = props.person;
        const register = props.register;

        const generator = certGenerator[courseN];
        if (generator === undefined) {
            console.error(`Generator function with key \`${courseN}\` not found`);
            return;
        }

        const {year, month, day} = dateComponents();
        const personFullName = `${person.person_names} ${person.person_paternal_surname} ${person.person_maternal_surname}`;

        generator(`${courseN} - ${personFullName}.docx`, {
            matpel: null,
            personFullName,
            personDni: person.person_dni.toString(),
            certCode: register.register_code.toString().padStart(4, "0"),
            certYear: year,
            certMonth: month,
            certDay: day,
            certIId: register.register_code,
        });
    };

    return (
        <div class="inline-block w-[12rem] p-1 border border-c-outline rounded-md">
            <button
                onclick={genCert}
                class="rounded-full bg-c-primary-container hover:bg-c-primary transition-colors h-12 w-12"
            >
                <DownloadIcon fill="var(--c-on-primary-container)" />
            </button>
            <div class="inline-block h-12 w-32 pl-2 align-middle">
                <p class="font-bold">
                    {courseName()}
                </p>
                <p class="font-mono text-sm">
                    {displayDate()}
                    &nbsp;-&nbsp;
                    {props.register.register_code}
                </p>
            </div>
        </div>
    );
}
